#ifndef _LABEL_H_
#define _LABEL_H_

#ifdef _LABEL_C_
#define E
#else
#define E extern
#endif

#ifndef Bitfield
#define Bitfield(x,n)	unsigned char x:n
#endif

#define HASHNUM 997
#define DEFLBUFSIZ 128
#define DEFSBUFSIZ 4096

/* type definition */
typedef struct {
	unsigned long	labelstr;	/* label string index */
	unsigned short	value;		/* label value */
	char		defstate;	/* defined? */
#define LABEL_UNDEFINED	0		/* undefined */
#define LABEL_DEFINED	1		/* fully defined */
#define LABEL_MACRO	2		/* defined as macro */
#define LABEL_UNRESSHOWN -1		/* already displayed as unresolved label */
	char		reftype;	/* reference type */
#define LF_NORMAL		0	/* normal */
#define LF_REDEFINABLE		0x01	/* re-definable */
#define LF_HIDE			0x02	/* not shown in label list */
#define LF_APPEARED_IN_PASS2	0x04	/* appeared in pass2 */
#define LF_GLOBAL		0x08	/* defined as global label (::) */
#define LF_PAGE_ALIGNED		0x10	/* will be placed at xx00h later */
#define LF_IN_PAGE		0x20	/* will be placed in page later */
	unsigned long	nextlabel;	/* next label index */
	long		value2;		/* sub info of this label */
} label;

#define MAXNAMESPACELEN 64
typedef struct label_container_ {
	label	*labelbuf;			/* label buffer */
	int	labelbufsiz;			/* size of the label buffer */
	int	nextlabelindex;			/* next label index to entry */
	struct	label_container_ *parent;	/* parent namespace */
	struct	label_container_ *next;		/* next label container */
	int	id;				/* ID of the namespace */
	unsigned long htbl[HASHNUM];		/* hash table */
	char	name[MAXNAMESPACELEN];		/* name of the label buffer */
} label_container;

typedef struct labellist_ {
	long	li;
	struct labellist_ *next;
} labellist;

#define DEFLNSBUFSIZ 64
typedef struct label_across_namespace_ {
	int	ns;
	long	li;
} label_across_namespace;

/* functions */
E int init_labeltable(void);
E void tini_labeltable(void);
E long getlabel(char *);
E long entrylabel(char *);
E char *getlabelstr(long);
E label *getlabelptr(long);
E label *getlabelptr_outside(char *, char);
E long entrylabel_across_namespace(char *, long);

E unsigned long add_string(char *);
E char *get_string(long);

E label_container *new_label_container(void);
E void dispose_label_containers(label_container *);
E int get_namespace_id(void);
E int switch_namespace(int);
E int dig_namespace(char *);
E void dig_namespace_pass2(void);
E int leave_namespace(void);
E void topmost_namespace(void);
E int next_namespace(void);
E char *getlabelstr_with_namespace(long);

#undef E

#endif /*_LABEL_H_*/
